@@ -20,34 +20,31 @@
  *                                                                         *
  ***************************************************************************/
 
-
 #include "PreCompiled.h"
 #ifndef _PreComp_
-# include <QEventLoop>
 # include <QCoreApplication>
-# include <QFileInfo>
-# include <QTimer>
+# include <QEventLoop>
 #endif
 
-#include "PythonDebugger.h"
-#include "MainWindow.h"
-#include "EditorView.h"
-#include "PythonEditor.h"
-#include "BitmapFactory.h"
-#include <Base/Interpreter.h>
 #include <Base/Console.h>
+#include <Base/Interpreter.h>
+
+#include "PythonDebugger.h"
+#include "BitmapFactory.h"
+#include "EditorView.h"
+#include "MainWindow.h"
+#include "PythonEditor.h"
+
 
 using namespace Gui;
 
-Breakpoint::Breakpoint()
-{
-}
+Breakpoint::Breakpoint() = default;
 
 Breakpoint::Breakpoint(const Breakpoint& rBp)
 {
     setFilename(rBp.filename());
-    for (std::set<int>::const_iterator it = rBp._linenums.begin(); it != rBp._linenums.end(); ++it)
-        _linenums.insert(*it);
+    for (int it : rBp._linenums)
+        _linenums.insert(it);
 }
 
 Breakpoint& Breakpoint::operator= (const Breakpoint& rBp)
@@ -56,15 +53,12 @@ Breakpoint& Breakpoint::operator= (const Breakpoint& rBp)
         return *this;
     setFilename(rBp.filename());
     _linenums.clear();
-    for (std::set<int>::const_iterator it = rBp._linenums.begin(); it != rBp._linenums.end(); ++it)
-        _linenums.insert(*it);
+    for (int it : rBp._linenums)
+        _linenums.insert(it);
     return *this;
 }
 
-Breakpoint::~Breakpoint()
-{
-
-}
+Breakpoint::~Breakpoint() = default;
 
 void Breakpoint::setFilename(const QString& fn)
 {
@@ -89,23 +83,22 @@ bool Breakpoint::checkLine(int line)
 int Breakpoint::lineIndex(int ind)const
 {
     int i = 0;
-    for (std::set<int>::const_iterator it = _linenums.begin(); it != _linenums.end(); ++it)
+    for (int it : _linenums)
     {
         if (ind == i++)
-            return *it;
+            return it;
     }
     return -1;
 }
 
 // -----------------------------------------------------
 
-void PythonDebugModule::init_module(void)
+void PythonDebugModule::init_module()
 {
     PythonDebugStdout::init_type();
     PythonDebugStderr::init_type();
     PythonDebugExcept::init_type();
-    static PythonDebugModule* mod = new PythonDebugModule();
-    Q_UNUSED(mod);
+    Base::Interpreter().addModule(new PythonDebugModule);
 }
 
 PythonDebugModule::PythonDebugModule()
@@ -131,24 +124,27 @@ PythonDebugModule::PythonDebugModule()
 
 PythonDebugModule::~PythonDebugModule()
 {
+    Py::Dict d(moduleDictionary());
+    d["StdOut"] = Py::None();
+    d["StdErr"] = Py::None();
 }
 
-Py::Object PythonDebugModule::getFunctionCallCount(const Py::Tuple &a)
+Py::Object PythonDebugModule::getFunctionCallCount(const Py::Tuple &)
 {
     return Py::None();
 }
 
-Py::Object PythonDebugModule::getExceptionCount(const Py::Tuple &a)
+Py::Object PythonDebugModule::getExceptionCount(const Py::Tuple &)
 {
     return Py::None();
 }
 
-Py::Object PythonDebugModule::getLineCount(const Py::Tuple &a)
+Py::Object PythonDebugModule::getLineCount(const Py::Tuple &)
 {
     return Py::None();
 }
 
-Py::Object PythonDebugModule::getFunctionReturnCount(const Py::Tuple &a)
+Py::Object PythonDebugModule::getFunctionReturnCount(const Py::Tuple &)
 {
     return Py::None();
 }
@@ -165,13 +161,9 @@ void PythonDebugStdout::init_type()
     add_varargs_method("flush",&PythonDebugStdout::flush,"flush the output");
 }
 
-PythonDebugStdout::PythonDebugStdout()
-{
-}
+PythonDebugStdout::PythonDebugStdout() = default;
 
-PythonDebugStdout::~PythonDebugStdout()
-{
-}
+PythonDebugStdout::~PythonDebugStdout() = default;
 
 Py::Object PythonDebugStdout::repr()
 {
@@ -184,9 +176,10 @@ Py::Object PythonDebugStdout::repr()
 Py::Object PythonDebugStdout::write(const Py::Tuple& args)
 {
     char *msg;
-    PyObject* pObj;
-    //args contains a single parameter which is the string to write.
-    if (!PyArg_ParseTuple(args.ptr(), "Os:OutputString", &pObj, &msg))
+    //PyObject* pObj;
+    ////args contains a single parameter which is the string to write.
+    //if (!PyArg_ParseTuple(args.ptr(), "Os:OutputString", &pObj, &msg))
+    if (!PyArg_ParseTuple(args.ptr(), "s:OutputString", &msg))
         throw Py::Exception();
 
     if (strlen(msg) > 0)
@@ -216,13 +209,9 @@ void PythonDebugStderr::init_type()
     add_varargs_method("write",&PythonDebugStderr::write,"write to stderr");
 }
 
-PythonDebugStderr::PythonDebugStderr()
-{
-}
+PythonDebugStderr::PythonDebugStderr() = default;
 
-PythonDebugStderr::~PythonDebugStderr()
-{
-}
+PythonDebugStderr::~PythonDebugStderr() = default;
 
 Py::Object PythonDebugStderr::repr()
 {
@@ -235,9 +224,10 @@ Py::Object PythonDebugStderr::repr()
 Py::Object PythonDebugStderr::write(const Py::Tuple& args)
 {
     char *msg;
-    PyObject* pObj;
+    //PyObject* pObj;
     //args contains a single parameter which is the string to write.
-    if (!PyArg_ParseTuple(args.ptr(), "Os:OutputDebugString", &pObj, &msg))
+    //if (!PyArg_ParseTuple(args.ptr(), "Os:OutputDebugString", &pObj, &msg))
+    if (!PyArg_ParseTuple(args.ptr(), "s:OutputDebugString", &msg))
         throw Py::Exception();
 
     if (strlen(msg) > 0)
@@ -247,6 +237,7 @@ Py::Object PythonDebugStderr::write(const Py::Tuple& args)
 
         //send it to the debugger as well
         //g_DebugSocket.SendMessage(eMSG_TRACE, msg);
+        Base::Console().Error("%s", msg);
     }
 
     return Py::None();
@@ -263,13 +254,9 @@ void PythonDebugExcept::init_type()
     add_varargs_method("fc_excepthook",&PythonDebugExcept::excepthook,"Custom exception handler");
 }
 
-PythonDebugExcept::PythonDebugExcept()
-{
-}
+PythonDebugExcept::PythonDebugExcept() = default;
 
-PythonDebugExcept::~PythonDebugExcept()
-{
-}
+PythonDebugExcept::~PythonDebugExcept() = default;
 
 Py::Object PythonDebugExcept::repr()
 {
@@ -298,7 +285,7 @@ Py::Object PythonDebugExcept::excepthook(const Py::Tuple& args)
             //get the pointer to the frame held by the bottom traceback object - this
             //should be where the exception occurred.
             tracebackobject* pTb = (tracebackobject*)tb;
-            while (pTb->tb_next != NULL) 
+            while (pTb->tb_next != NULL)
             {
                 pTb = pTb->tb_next;
             }
@@ -313,11 +300,11 @@ Py::Object PythonDebugExcept::excepthook(const Py::Tuple& args)
 // -----------------------------------------------------
 
 namespace Gui {
-class PythonDebuggerPy : public Py::PythonExtension<PythonDebuggerPy> 
+class PythonDebuggerPy : public Py::PythonExtension<PythonDebuggerPy>
 {
 public:
-    PythonDebuggerPy(PythonDebugger* d) : dbg(d), depth(0) { }
-    ~PythonDebuggerPy() {}
+    explicit PythonDebuggerPy(PythonDebugger* d) : dbg(d), depth(0) { }
+    ~PythonDebuggerPy() override = default;
     PythonDebugger* dbg;
     int depth;
 };
@@ -325,7 +312,7 @@ public:
 class RunningState
 {
 public:
-    RunningState(bool& s) : state(s)
+    explicit RunningState(bool& s) : state(s)
     { state = true; }
     ~RunningState()
     { state = false; }
@@ -334,34 +321,35 @@ private:
 };
 
 struct PythonDebuggerP {
-    PyObject* out_o;
-    PyObject* err_o;
-    PyObject* exc_o;
-    PyObject* out_n;
-    PyObject* err_n;
-    PyObject* exc_n;
-    bool init, trystop, running;
+    PyObject* out_o{nullptr};
+    PyObject* err_o{nullptr};
+    PyObject* exc_o{nullptr};
+    PyObject* out_n{nullptr};
+    PyObject* err_n{nullptr};
+    PyObject* exc_n{nullptr};
+    PythonDebugExcept* pypde;
+    bool init{false}, trystop{false}, running{false};
     QEventLoop loop;
-    PyObject* pydbg;
+    PyObject* pydbg{nullptr};
     std::vector<Breakpoint> bps;
 
-    PythonDebuggerP(PythonDebugger* that) :
-        init(false), trystop(false), running(false)
+    explicit PythonDebuggerP(PythonDebugger* that)
     {
         Base::PyGILStateLocker lock;
         out_n = new PythonDebugStdout();
         err_n = new PythonDebugStderr();
-        PythonDebugExcept* err = new PythonDebugExcept();
-        Py::Object func = err->getattr("fc_excepthook");
+        pypde = new PythonDebugExcept();
+        Py::Object func = pypde->getattr("fc_excepthook");
         exc_n = Py::new_reference_to(func);
-        Py_DECREF(err);
         pydbg = new PythonDebuggerPy(that);
     }
     ~PythonDebuggerP()
     {
+        Base::PyGILStateLocker lock;
         Py_DECREF(out_n);
         Py_DECREF(err_n);
         Py_DECREF(exc_n);
+        Py_DECREF(pypde);
         Py_DECREF(pydbg);
     }
 };
@@ -379,25 +367,25 @@ PythonDebugger::~PythonDebugger()
 
 Breakpoint PythonDebugger::getBreakpoint(const QString& fn) const
 {
-    for (std::vector<Breakpoint>::const_iterator it = d->bps.begin(); it != d->bps.end(); ++it) {
-        if (fn == it->filename()) {
-            return *it;
+    for (const Breakpoint& it : d->bps) {
+        if (fn == it.filename()) {
+            return it;
         }
     }
 
-    return Breakpoint();
+    return {};
 }
 
 bool PythonDebugger::toggleBreakpoint(int line, const QString& fn)
 {
-    for (std::vector<Breakpoint>::iterator it = d->bps.begin(); it != d->bps.end(); ++it) {
-        if (fn == it->filename()) {
-            if (it->checkLine(line)) {
-                it->removeLine(line);
+    for (Breakpoint& it : d->bps) {
+        if (fn == it.filename()) {
+            if (it.checkLine(line)) {
+                it.removeLine(line);
                 return false;
             }
             else {
-                it->addLine(line);
+                it.addLine(line);
                 return true;
             }
         }
@@ -414,9 +402,46 @@ void PythonDebugger::runFile(const QString& fn)
 {
     try {
         RunningState state(d->running);
-        Base::Interpreter().runFile((const char*)fn.toUtf8());
+        QByteArray pxFileName = fn.toUtf8();
+#ifdef FC_OS_WIN32
+        Base::FileInfo fi((const char*)pxFileName);
+        FILE *fp = _wfopen(fi.toStdWString().c_str(),L"r");
+#else
+        FILE *fp = fopen((const char*)pxFileName,"r");
+#endif
+        if (!fp)
+            return;
+
+        Base::PyGILStateLocker locker;
+        PyObject *module, *dict;
+        module = PyImport_AddModule("__main__");
+        dict = PyModule_GetDict(module);
+        dict = PyDict_Copy(dict);
+        if (!PyDict_GetItemString(dict, "__file__")) {
+            PyObject *pyObj = PyUnicode_FromString((const char*)pxFileName);
+            if (!pyObj) {
+                fclose(fp);
+                return;
+            }
+            if (PyDict_SetItemString(dict, "__file__", pyObj) < 0) {
+                Py_DECREF(pyObj);
+                fclose(fp);
+                return;
+            }
+            Py_DECREF(pyObj);
+        }
+
+        PyObject *result = PyRun_File(fp, (const char*)pxFileName, Py_file_input, dict, dict);
+        fclose(fp);
+        Py_DECREF(dict);
+
+        if (!result)
+            PyErr_Print();
+        else
+            Py_DECREF(result);
     }
-    catch (const Base::PyException&) {
+    catch (const Base::PyException&/* e*/) {
+        //PySys_WriteStderr("Exception: %s\n", e.what());
     }
     catch (...) {
         Base::Console().Warning("Unknown exception thrown during macro debugging\n");
@@ -452,7 +477,7 @@ bool PythonDebugger::stop()
     if (!d->init)
         return false;
     Base::PyGILStateLocker lock;
-    PyEval_SetTrace(NULL, NULL);
+    PyEval_SetTrace(nullptr, nullptr);
     PySys_SetObject("stdout", d->out_o);
     PySys_SetObject("stderr", d->err_o);
     PySys_SetObject("excepthook", d->exc_o);
@@ -463,27 +488,37 @@ bool PythonDebugger::stop()
 void PythonDebugger::tryStop()
 {
     d->trystop = true;
-    signalNextStep();
+    Q_EMIT signalNextStep();
 }
 
 void PythonDebugger::stepOver()
 {
-    signalNextStep();
+    Q_EMIT signalNextStep();
+}
+
+void PythonDebugger::stepInto()
+{
+    Q_EMIT signalNextStep();
+}
+
+void PythonDebugger::stepRun()
+{
+    Q_EMIT signalNextStep();
 }
 
 void PythonDebugger::showDebugMarker(const QString& fn, int line)
 {
-    PythonEditorView* edit = 0;
+    PythonEditorView* edit = nullptr;
     QList<QWidget*> mdis = getMainWindow()->windows();
-    for (QList<QWidget*>::iterator it = mdis.begin(); it != mdis.end(); ++it) {
-        edit = qobject_cast<PythonEditorView*>(*it);
+    for (const auto & mdi : mdis) {
+        edit = qobject_cast<PythonEditorView*>(mdi);
         if (edit && edit->fileName() == fn)
             break;
     }
 
     if (!edit) {
-        PythonEditor* editor = new PythonEditor();
-        editor->setWindowIcon(Gui::BitmapFactory().pixmap("python_small"));
+        auto editor = new PythonEditor();
+        editor->setWindowIcon(Gui::BitmapFactory().iconFromTheme("applications-python"));
         edit = new PythonEditorView(editor, getMainWindow());
         edit->open(fn);
         edit->resize(400, 300);
@@ -496,10 +531,10 @@ void PythonDebugger::showDebugMarker(const QString& fn, int line)
 
 void PythonDebugger::hideDebugMarker(const QString& fn)
 {
-    PythonEditorView* edit = 0;
+    PythonEditorView* edit = nullptr;
     QList<QWidget*> mdis = getMainWindow()->windows();
-    for (QList<QWidget*>::iterator it = mdis.begin(); it != mdis.end(); ++it) {
-        edit = qobject_cast<PythonEditorView*>(*it);
+    for (const auto & mdi : mdis) {
+        edit = qobject_cast<PythonEditorView*>(mdi);
         if (edit && edit->fileName() == fn) {
             edit->hideDebugMarker();
             break;
@@ -507,22 +542,28 @@ void PythonDebugger::hideDebugMarker(const QString& fn)
     }
 }
 
+#if PY_VERSION_HEX < 0x030900B1
+static PyCodeObject* PyFrame_GetCode(PyFrameObject *frame)
+{
+    Py_INCREF(frame->f_code);
+    return frame->f_code;
+}
+#endif
+
 // http://www.koders.com/cpp/fidBA6CD8A0FE5F41F1464D74733D9A711DA257D20B.aspx?s=PyEval_SetTrace
 // http://code.google.com/p/idapython/source/browse/trunk/python.cpp
 // http://www.koders.com/cpp/fid191F7B13CF73133935A7A2E18B7BF43ACC6D1784.aspx?s=PyEval_SetTrace
-// http://stuff.mit.edu/afs/sipb/project/python/src/python2.2-2.2.2/Modules/_hotshot.c
-int PythonDebugger::tracer_callback(PyObject *obj, PyFrameObject *frame, int what, PyObject *arg)
+
+int PythonDebugger::tracer_callback(PyObject *obj, PyFrameObject *frame, int what, PyObject * /*arg*/)
 {
-    PythonDebuggerPy* self = static_cast<PythonDebuggerPy*>(obj);
+    auto self = static_cast<PythonDebuggerPy*>(obj);
     PythonDebugger* dbg = self->dbg;
     if (dbg->d->trystop)
         PyErr_SetInterrupt();
     QCoreApplication::processEvents();
-    //int no;
-
-    //no = frame->f_tstate->recursion_depth;
-    //char* name = PyString_AsString(frame->f_code->co_name);
-    QString file = QString::fromUtf8(PyString_AsString(frame->f_code->co_filename));
+    PyCodeObject* code = PyFrame_GetCode(frame);
+    QString file = QString::fromUtf8(PyUnicode_AsUTF8(code->co_filename));
+    Py_DECREF(code);
     switch (what) {
     case PyTrace_CALL:
         self->depth++;
@@ -533,21 +574,17 @@ int PythonDebugger::tracer_callback(PyObject *obj, PyFrameObject *frame, int wha
         return 0;
     case PyTrace_LINE:
         {
-            //PyObject *str;
-            //str = PyObject_Str(frame->f_code->co_filename);
-            //no = frame->f_lineno;
-            int line = PyCode_Addr2Line(frame->f_code, frame->f_lasti);
-            //if (str) {
-            //    Base::Console().Message("PROFILING: %s:%d\n", PyString_AsString(str), frame->f_lineno);
-            //    Py_DECREF(str);
-            //}
-    // For testing only
+            PyCodeObject* f_code = PyFrame_GetCode(frame);
+            int f_lasti = PyFrame_GetLineNumber(frame);
+            int line = PyCode_Addr2Line(f_code, f_lasti);
+            Py_DECREF(f_code);
+
             if (!dbg->d->trystop) {
                 Breakpoint bp = dbg->getBreakpoint(file);
                 if (bp.checkLine(line)) {
                     dbg->showDebugMarker(file, line);
                     QEventLoop loop;
-                    QObject::connect(dbg, SIGNAL(signalNextStep()), &loop, SLOT(quit()));
+                    QObject::connect(dbg, &PythonDebugger::signalNextStep, &loop, &QEventLoop::quit);
                     loop.exec();
                     dbg->hideDebugMarker(file);
                 }
