@@ -20,46 +20,102 @@
  *                                                                         *
  ***************************************************************************/
 
-
 #ifndef GUI_SPLITVIEW3DINVENTOR_H
 #define GUI_SPLITVIEW3DINVENTOR_H
 
 #include "MDIView.h"
+#include "MDIViewPy.h"
 
 #include <Base/Parameter.h>
 #include <vector>
 
+
 namespace Gui {
 class View3DInventorViewer;
-
+class AbstractSplitViewPy;
+class View3DSettings;
 
 /** The SplitView3DInventor class allows to create a window with two or more Inventor views.
  *  \author Werner Mayer
  */
-class GuiExport SplitView3DInventor : public MDIView,public ParameterGrp::ObserverType
+class GuiExport AbstractSplitView : public MDIView
 {
+    Q_OBJECT
+
+    TYPESYSTEM_HEADER_WITH_OVERRIDE();
+
 public:
-    SplitView3DInventor(int views, Gui::Document* pcDocument, QWidget* parent, Qt::WFlags wflags=0);
-    ~SplitView3DInventor();
+    AbstractSplitView(Gui::Document* pcDocument, QWidget* parent, Qt::WindowFlags wflags=Qt::WindowFlags());
+    ~AbstractSplitView() override;
 
-    virtual const char *getName(void) const;
+    const char *getName() const override;
 
-    /// Mesage handler
-    virtual bool onMsg(const char* pMsg, const char** ppReturn);
-    virtual bool onHasMsg(const char* pMsg) const;
-    virtual void OnChange(ParameterGrp::SubjectType &rCaller,ParameterGrp::MessageType Reason);
-    virtual void onUpdate(void);
+    /// Message handler
+    bool onMsg(const char* pMsg, const char** ppReturn) override;
+    bool onHasMsg(const char* pMsg) const override;
+    void onUpdate() override;
+    void deleteSelf() override;
+    void viewAll() override;
 
     View3DInventorViewer *getViewer(unsigned int) const;
+    void setOverrideCursor(const QCursor&) override;
+    bool containsViewProvider(const ViewProvider*) const override;
 
-    void setCursor(const QCursor&);
+    PyObject *getPyObject() override;
+    void setPyObject(PyObject *) override;
+    int getSize();
 
 protected:
-    /// handle to the viewer parameter group
-    ParameterGrp::handle hGrp;
+    void setDocumentOfViewers(Gui::Document* document);
+    void setupSettings();
+
+protected:
+    std::vector<View3DInventorViewer*> _viewer;
+    PyObject *_viewerPy;
+    std::unique_ptr<View3DSettings> viewSettings;
+};
+
+class AbstractSplitViewPy : public Py::PythonExtension<AbstractSplitViewPy>
+{
+public:
+    using BaseType = Py::PythonExtension<AbstractSplitViewPy>;
+    static void init_type();    // announce properties and methods
+
+    AbstractSplitViewPy(AbstractSplitView *vi);
+    ~AbstractSplitViewPy() override;
+    AbstractSplitView* getSplitViewPtr();
+
+    Py::Object repr() override;
+    Py::Object getattr(const char *) override;
+    Py::Object cast_to_base(const Py::Tuple&);
+
+    Py::Object fitAll(const Py::Tuple&);
+    Py::Object viewBottom(const Py::Tuple&);
+    Py::Object viewFront(const Py::Tuple&);
+    Py::Object viewLeft(const Py::Tuple&);
+    Py::Object viewRear(const Py::Tuple&);
+    Py::Object viewRight(const Py::Tuple&);
+    Py::Object viewTop(const Py::Tuple&);
+    Py::Object viewIsometric(const Py::Tuple&);
+    Py::Object getViewer(const Py::Tuple&);
+    Py::Object sequence_item(Py_ssize_t) override;
+    Py::Object close(const Py::Tuple&);
+    PyCxx_ssize_t sequence_length() override;
 
 private:
-    std::vector<View3DInventorViewer*> _viewer;
+    Gui::MDIViewPy base;
+};
+
+/** The SplitView3DInventor class allows to create a window with two or more Inventor views.
+ *  \author Werner Mayer
+ */
+class GuiExport SplitView3DInventor : public AbstractSplitView
+{
+    TYPESYSTEM_HEADER_WITH_OVERRIDE();
+
+public:
+    SplitView3DInventor(int views, Gui::Document* pcDocument, QWidget* parent, Qt::WindowFlags wflags=Qt::WindowFlags());
+    ~SplitView3DInventor() override;
 };
 
 } // namespace Gui
