@@ -20,38 +20,28 @@
  *                                                                         *
  ***************************************************************************/
 
-
 #include "PreCompiled.h"
-
 #ifndef _PreComp_
-# include <Inventor/nodes/SoBaseColor.h>
-# include <Inventor/nodes/SoCoordinate3.h>
-# include <Inventor/nodes/SoDrawStyle.h>
-# include <Inventor/nodes/SoFaceSet.h>
-# include <Inventor/nodes/SoLineSet.h>
-# include <Inventor/nodes/SoMarkerSet.h>
-# include <Inventor/nodes/SoSeparator.h>
-# include <Inventor/nodes/SoShapeHints.h>
+#include <Inventor/nodes/SoBaseColor.h>
+#include <Inventor/nodes/SoCoordinate3.h>
+#include <Inventor/nodes/SoDrawStyle.h>
+#include <Inventor/nodes/SoFaceSet.h>
+#include <Inventor/nodes/SoLineSet.h>
+#include <Inventor/nodes/SoMarkerSet.h>
+#include <Inventor/nodes/SoSeparator.h>
+#include <Inventor/nodes/SoShapeHints.h>
 #endif
 
-/// Here the FreeCAD includes sorted by Base,App,Gui......
-#include <Base/Console.h>
-#include <Base/Parameter.h>
-#include <Base/Exception.h>
-#include <Base/Sequencer.h>
 #include <App/Application.h>
-#include <Gui/Selection.h>
-#include <Gui/SoFCSelection.h>
-
-#include <Mod/Mesh/App/Core/Degeneration.h>
-#include <Mod/Mesh/App/Core/Evaluation.h>
-#include <Mod/Mesh/App/Core/Iterator.h>
-#include <Mod/Mesh/App/Mesh.h>
+#include <Base/Parameter.h>
+#include <Gui/Inventor/MarkerBitmaps.h>
 #include <Mod/Mesh/App/MeshFeature.h>
+#include <Mod/Mesh/App/Core/Degeneration.h>
+#include <Mod/Mesh/App/Core/Iterator.h>
 
-#include "ViewProvider.h"
 #include "ViewProviderDefects.h"
 
+
 using namespace Mesh;
 using namespace MeshGui;
 
@@ -59,6 +49,7 @@ using namespace MeshGui;
 PROPERTY_SOURCE_ABSTRACT(MeshGui::ViewProviderMeshDefects, Gui::ViewProviderDocumentObject)
 PROPERTY_SOURCE(MeshGui::ViewProviderMeshOrientation, MeshGui::ViewProviderMeshDefects)
 PROPERTY_SOURCE(MeshGui::ViewProviderMeshNonManifolds, MeshGui::ViewProviderMeshDefects)
+PROPERTY_SOURCE(MeshGui::ViewProviderMeshNonManifoldPoints, MeshGui::ViewProviderMeshDefects)
 PROPERTY_SOURCE(MeshGui::ViewProviderMeshDuplicatedFaces, MeshGui::ViewProviderMeshDefects)
 PROPERTY_SOURCE(MeshGui::ViewProviderMeshDuplicatedPoints, MeshGui::ViewProviderMeshDefects)
 PROPERTY_SOURCE(MeshGui::ViewProviderMeshDegenerations, MeshGui::ViewProviderMeshDefects)
@@ -68,7 +59,7 @@ PROPERTY_SOURCE(MeshGui::ViewProviderMeshFolds, MeshGui::ViewProviderMeshDefects
 
 ViewProviderMeshDefects::ViewProviderMeshDefects()
 {
-    ADD_PROPERTY(LineWidth,(2.0f));
+    ADD_PROPERTY(LineWidth, (2.0F));
 
     pcCoords = new SoCoordinate3();
     pcCoords->ref();
@@ -89,6 +80,12 @@ void ViewProviderMeshDefects::onChanged(const App::Property* prop)
     if (prop == &LineWidth) {
         pcDrawStyle->lineWidth = LineWidth.getValue();
     }
+    // Visibility changes must be handled here because in the base class it changes the attribute of
+    // the feature and thus affects the visibility of the mesh view provider which is undesired
+    // behaviour
+    else if (prop == &Visibility) {
+        Visibility.getValue() ? show() : hide();
+    }
     else {
         ViewProviderDocumentObject::onChanged(prop);
     }
@@ -98,8 +95,10 @@ void ViewProviderMeshDefects::onChanged(const App::Property* prop)
 
 ViewProviderMeshOrientation::ViewProviderMeshOrientation()
 {
+    // NOLINTBEGIN
     pcFaces = new SoFaceSet;
     pcFaces->ref();
+    // NOLINTEND
 }
 
 ViewProviderMeshOrientation::~ViewProviderMeshOrientation()
@@ -109,7 +108,7 @@ ViewProviderMeshOrientation::~ViewProviderMeshOrientation()
 
 void ViewProviderMeshOrientation::attach(App::DocumentObject* pcFeat)
 {
-    ViewProviderDocumentObject::attach( pcFeat );
+    ViewProviderDocumentObject::attach(pcFeat);
 
     SoGroup* pcFaceRoot = new SoGroup();
 
@@ -117,24 +116,28 @@ void ViewProviderMeshOrientation::attach(App::DocumentObject* pcFeat)
     pcFlatStyle->style = SoDrawStyle::FILLED;
     pcFaceRoot->addChild(pcFlatStyle);
 
-    SoShapeHints * flathints = new SoShapeHints;
-    flathints->vertexOrdering = SoShapeHints::COUNTERCLOCKWISE ;
+    SoShapeHints* flathints = new SoShapeHints;
+    flathints->vertexOrdering = SoShapeHints::COUNTERCLOCKWISE;
     flathints->shapeType = SoShapeHints::UNKNOWN_SHAPE_TYPE;
     pcFaceRoot->addChild(flathints);
 
     // Draw faces
     SoSeparator* linesep = new SoSeparator;
-    SoBaseColor * basecol = new SoBaseColor;
-    basecol->rgb.setValue( 1.0f, 0.5f, 0.0f );
+    SoBaseColor* basecol = new SoBaseColor;
+    basecol->rgb.setValue(1.0F, 0.5F, 0.0F);
     linesep->addChild(basecol);
     linesep->addChild(pcCoords);
     linesep->addChild(pcFaces);
 
     // Draw markers
-    SoBaseColor * markcol = new SoBaseColor;
-    markcol->rgb.setValue( 1.0f, 1.0f, 0.0f );
+    SoBaseColor* markcol = new SoBaseColor;
+    markcol->rgb.setValue(1.0F, 1.0F, 0.0F);
     SoMarkerSet* marker = new SoMarkerSet;
-    marker->markerIndex=SoMarkerSet::PLUS_7_7;
+    marker->markerIndex = Gui::Inventor::MarkerBitmaps::getMarkerIndex(
+        "PLUS",
+        App::GetApplication()
+            .GetParameterGroupByPath("User parameter:BaseApp/Preferences/View")
+            ->GetInt("MarkerSize", 7));
     linesep->addChild(markcol);
     linesep->addChild(marker);
 
@@ -143,23 +146,22 @@ void ViewProviderMeshOrientation::attach(App::DocumentObject* pcFeat)
     addDisplayMaskMode(pcFaceRoot, "Face");
 }
 
-void ViewProviderMeshOrientation::showDefects(const std::vector<unsigned long>& inds)
+void ViewProviderMeshOrientation::showDefects(const std::vector<Mesh::ElementIndex>& inds)
 {
-    Mesh::Feature* f = dynamic_cast<Mesh::Feature*>(pcObject);
-    const MeshCore::MeshKernel & rMesh = f->Mesh.getValue().getKernel();
+    Mesh::Feature* f = static_cast<Mesh::Feature*>(pcObject);
+    const MeshCore::MeshKernel& rMesh = f->Mesh.getValue().getKernel();
 
     pcCoords->point.deleteValues(0);
-    pcCoords->point.setNum(3*inds.size());
+    pcCoords->point.setNum(3 * inds.size());
     MeshCore::MeshFacetIterator cF(rMesh);
-    unsigned long i=0;
-    unsigned long j=0;
-    for (std::vector<unsigned long>::const_iterator it = inds.begin(); it != inds.end(); ++it) {
-        cF.Set(*it);
-        for (int k=0; k<3; k++) {
-            Base::Vector3f cP = cF->_aclPoints[k];
+    int i = 0;
+    int j = 0;
+    for (Mesh::ElementIndex ind : inds) {
+        cF.Set(ind);
+        for (auto cP : cF->_aclPoints) {
             // move a bit in opposite normal direction to overlay the original faces
-            cP -= 0.001f * cF->GetNormal();
-            pcCoords->point.set1Value(i++,cP.x,cP.y,cP.z);
+            cP -= 0.001F * cF->GetNormal();
+            pcCoords->point.set1Value(i++, cP.x, cP.y, cP.z);
         }
         pcFaces->numVertices.set1Value(j++, 3);
     }
@@ -171,8 +173,10 @@ void ViewProviderMeshOrientation::showDefects(const std::vector<unsigned long>&
 
 ViewProviderMeshNonManifolds::ViewProviderMeshNonManifolds()
 {
+    // NOLINTBEGIN
     pcLines = new SoLineSet;
     pcLines->ref();
+    // NOLINTEND
 }
 
 ViewProviderMeshNonManifolds::~ViewProviderMeshNonManifolds()
@@ -182,7 +186,7 @@ ViewProviderMeshNonManifolds::~ViewProviderMeshNonManifolds()
 
 void ViewProviderMeshNonManifolds::attach(App::DocumentObject* pcFeat)
 {
-    ViewProviderDocumentObject::attach( pcFeat );
+    ViewProviderDocumentObject::attach(pcFeat);
 
     SoGroup* pcLineRoot = new SoGroup();
     pcDrawStyle->lineWidth = 3;
@@ -190,42 +194,48 @@ void ViewProviderMeshNonManifolds::attach(App::DocumentObject* pcFeat)
 
     // Draw lines
     SoSeparator* linesep = new SoSeparator;
-    SoBaseColor * basecol = new SoBaseColor;
-    basecol->rgb.setValue( 1.0f, 0.0f, 0.0f );
+    SoBaseColor* basecol = new SoBaseColor;
+    basecol->rgb.setValue(1.0F, 0.0F, 0.0F);
     linesep->addChild(basecol);
     linesep->addChild(pcCoords);
     linesep->addChild(pcLines);
     pcLineRoot->addChild(linesep);
 
     // Draw markers
-    SoBaseColor * markcol = new SoBaseColor;
-    markcol->rgb.setValue( 1.0f, 1.0f, 0.0f );
+    SoBaseColor* markcol = new SoBaseColor;
+    markcol->rgb.setValue(1.0F, 1.0F, 0.0F);
     SoMarkerSet* marker = new SoMarkerSet;
-    marker->markerIndex=SoMarkerSet::PLUS_7_7;
+    marker->markerIndex = Gui::Inventor::MarkerBitmaps::getMarkerIndex(
+        "PLUS",
+        App::GetApplication()
+            .GetParameterGroupByPath("User parameter:BaseApp/Preferences/View")
+            ->GetInt("MarkerSize", 7));
     linesep->addChild(markcol);
     linesep->addChild(marker);
 
     addDisplayMaskMode(pcLineRoot, "Line");
 }
 
-void ViewProviderMeshNonManifolds::showDefects(const std::vector<unsigned long>& inds)
+void ViewProviderMeshNonManifolds::showDefects(const std::vector<Mesh::ElementIndex>& inds)
 {
-    if ((inds.size() % 2) != 0)
+    if ((inds.size() % 2) != 0) {
         return;
-    Mesh::Feature* f = dynamic_cast<Mesh::Feature*>(pcObject);
-    const MeshCore::MeshKernel & rMesh = f->Mesh.getValue().getKernel();
+    }
+    Mesh::Feature* f = static_cast<Mesh::Feature*>(pcObject);
+    const MeshCore::MeshKernel& rMesh = f->Mesh.getValue().getKernel();
 
     pcCoords->point.deleteValues(0);
     pcCoords->point.setNum(inds.size());
     MeshCore::MeshPointIterator cP(rMesh);
-    unsigned long i=0;
-    unsigned long j=0;
-    for (std::vector<unsigned long>::const_iterator it = inds.begin(); it != inds.end(); ++it) {
+    int i = 0;
+    int j = 0;
+    for (std::vector<Mesh::ElementIndex>::const_iterator it = inds.begin(); it != inds.end();
+         ++it) {
         cP.Set(*it);
-        pcCoords->point.set1Value(i++,cP->x,cP->y,cP->z);
-        ++it; // go to end point
+        pcCoords->point.set1Value(i++, cP->x, cP->y, cP->z);
+        ++it;  // go to end point
         cP.Set(*it);
-        pcCoords->point.set1Value(i++,cP->x,cP->y,cP->z);
+        pcCoords->point.set1Value(i++, cP->x, cP->y, cP->z);
         pcLines->numVertices.set1Value(j++, 2);
     }
 
@@ -234,10 +244,75 @@ void ViewProviderMeshNonManifolds::showDefects(const std::vector<unsigned long>&
 
 // ----------------------------------------------------------------------
 
+ViewProviderMeshNonManifoldPoints::ViewProviderMeshNonManifoldPoints()
+{
+    // NOLINTBEGIN
+    pcPoints = new SoPointSet;
+    pcPoints->ref();
+    // NOLINTEND
+}
+
+ViewProviderMeshNonManifoldPoints::~ViewProviderMeshNonManifoldPoints()
+{
+    pcPoints->unref();
+}
+
+void ViewProviderMeshNonManifoldPoints::attach(App::DocumentObject* pcFeat)
+{
+    ViewProviderDocumentObject::attach(pcFeat);
+
+    SoGroup* pcPointRoot = new SoGroup();
+    pcDrawStyle->pointSize = 3;
+    pcPointRoot->addChild(pcDrawStyle);
+
+    // Draw points
+    SoSeparator* pointsep = new SoSeparator;
+    SoBaseColor* basecol = new SoBaseColor;
+    basecol->rgb.setValue(1.0F, 0.5F, 0.0F);
+    pointsep->addChild(basecol);
+    pointsep->addChild(pcCoords);
+    pointsep->addChild(pcPoints);
+    pcPointRoot->addChild(pointsep);
+
+    // Draw markers
+    SoBaseColor* markcol = new SoBaseColor;
+    markcol->rgb.setValue(1.0F, 1.0F, 0.0F);
+    SoMarkerSet* marker = new SoMarkerSet;
+    marker->markerIndex = Gui::Inventor::MarkerBitmaps::getMarkerIndex(
+        "PLUS",
+        App::GetApplication()
+            .GetParameterGroupByPath("User parameter:BaseApp/Preferences/View")
+            ->GetInt("MarkerSize", 7));
+    pointsep->addChild(markcol);
+    pointsep->addChild(marker);
+
+    addDisplayMaskMode(pcPointRoot, "Point");
+}
+
+void ViewProviderMeshNonManifoldPoints::showDefects(const std::vector<Mesh::ElementIndex>& inds)
+{
+    Mesh::Feature* f = static_cast<Mesh::Feature*>(pcObject);
+    const MeshCore::MeshKernel& rMesh = f->Mesh.getValue().getKernel();
+    pcCoords->point.deleteValues(0);
+    pcCoords->point.setNum(inds.size());
+    MeshCore::MeshPointIterator cP(rMesh);
+    int i = 0;
+    for (Mesh::ElementIndex ind : inds) {
+        cP.Set(ind);
+        pcCoords->point.set1Value(i++, cP->x, cP->y, cP->z);
+    }
+
+    setDisplayMaskMode("Point");
+}
+
+// ----------------------------------------------------------------------
+
 ViewProviderMeshDuplicatedFaces::ViewProviderMeshDuplicatedFaces()
 {
+    // NOLINTBEGIN
     pcFaces = new SoFaceSet;
     pcFaces->ref();
+    // NOLINTEND
 }
 
 ViewProviderMeshDuplicatedFaces::~ViewProviderMeshDuplicatedFaces()
@@ -247,7 +322,7 @@ ViewProviderMeshDuplicatedFaces::~ViewProviderMeshDuplicatedFaces()
 
 void ViewProviderMeshDuplicatedFaces::attach(App::DocumentObject* pcFeat)
 {
-    ViewProviderDocumentObject::attach( pcFeat );
+    ViewProviderDocumentObject::attach(pcFeat);
 
     SoGroup* pcFaceRoot = new SoGroup();
 
@@ -255,48 +330,51 @@ void ViewProviderMeshDuplicatedFaces::attach(App::DocumentObject* pcFeat)
     pcFlatStyle->style = SoDrawStyle::FILLED;
     pcFaceRoot->addChild(pcFlatStyle);
 
-    SoShapeHints * flathints = new SoShapeHints;
-    flathints->vertexOrdering = SoShapeHints::COUNTERCLOCKWISE ;
+    SoShapeHints* flathints = new SoShapeHints;
+    flathints->vertexOrdering = SoShapeHints::COUNTERCLOCKWISE;
     flathints->shapeType = SoShapeHints::UNKNOWN_SHAPE_TYPE;
     pcFaceRoot->addChild(flathints);
 
     // Draw lines
     SoSeparator* linesep = new SoSeparator;
-    SoBaseColor * basecol = new SoBaseColor;
-    basecol->rgb.setValue( 1.0f, 0.0f, 0.0f );
+    SoBaseColor* basecol = new SoBaseColor;
+    basecol->rgb.setValue(1.0F, 0.0F, 0.0F);
     linesep->addChild(basecol);
     linesep->addChild(pcCoords);
     linesep->addChild(pcFaces);
     pcFaceRoot->addChild(linesep);
 
     // Draw markers
-    SoBaseColor * markcol = new SoBaseColor;
-    markcol->rgb.setValue( 1.0f, 1.0f, 0.0f );
+    SoBaseColor* markcol = new SoBaseColor;
+    markcol->rgb.setValue(1.0F, 1.0F, 0.0F);
     SoMarkerSet* marker = new SoMarkerSet;
-    marker->markerIndex=SoMarkerSet::PLUS_7_7;
+    marker->markerIndex = Gui::Inventor::MarkerBitmaps::getMarkerIndex(
+        "PLUS",
+        App::GetApplication()
+            .GetParameterGroupByPath("User parameter:BaseApp/Preferences/View")
+            ->GetInt("MarkerSize", 7));
     linesep->addChild(markcol);
     linesep->addChild(marker);
 
     addDisplayMaskMode(pcFaceRoot, "Face");
 }
 
-void ViewProviderMeshDuplicatedFaces::showDefects(const std::vector<unsigned long>& inds)
+void ViewProviderMeshDuplicatedFaces::showDefects(const std::vector<Mesh::ElementIndex>& inds)
 {
-    Mesh::Feature* f = dynamic_cast<Mesh::Feature*>(pcObject);
-    const MeshCore::MeshKernel & rMesh = f->Mesh.getValue().getKernel();
+    Mesh::Feature* f = static_cast<Mesh::Feature*>(pcObject);
+    const MeshCore::MeshKernel& rMesh = f->Mesh.getValue().getKernel();
 
     pcCoords->point.deleteValues(0);
-    pcCoords->point.setNum(3*inds.size());
+    pcCoords->point.setNum(3 * inds.size());
     MeshCore::MeshFacetIterator cF(rMesh);
-    unsigned long i=0;
-    unsigned long j=0;
-    for (std::vector<unsigned long>::const_iterator it = inds.begin(); it != inds.end(); ++it) {
-        cF.Set(*it);
-        for (int k=0; k<3; k++) {
-            Base::Vector3f cP = cF->_aclPoints[k];
+    int i = 0;
+    int j = 0;
+    for (Mesh::ElementIndex ind : inds) {
+        cF.Set(ind);
+        for (auto cP : cF->_aclPoints) {
             // move a bit in normal direction to overlay the original faces
-            cP += 0.001f * cF->GetNormal();
-            pcCoords->point.set1Value(i++,cP.x,cP.y,cP.z);
+            cP += 0.001F * cF->GetNormal();
+            pcCoords->point.set1Value(i++, cP.x, cP.y, cP.z);
         }
         pcFaces->numVertices.set1Value(j++, 3);
     }
@@ -308,8 +386,10 @@ void ViewProviderMeshDuplicatedFaces::showDefects(const std::vector<unsigned lon
 
 ViewProviderMeshDuplicatedPoints::ViewProviderMeshDuplicatedPoints()
 {
+    // NOLINTBEGIN
     pcPoints = new SoPointSet;
     pcPoints->ref();
+    // NOLINTEND
 }
 
 ViewProviderMeshDuplicatedPoints::~ViewProviderMeshDuplicatedPoints()
@@ -319,7 +399,7 @@ ViewProviderMeshDuplicatedPoints::~ViewProviderMeshDuplicatedPoints()
 
 void ViewProviderMeshDuplicatedPoints::attach(App::DocumentObject* pcFeat)
 {
-    ViewProviderDocumentObject::attach( pcFeat );
+    ViewProviderDocumentObject::attach(pcFeat);
 
     SoGroup* pcPointRoot = new SoGroup();
     pcDrawStyle->pointSize = 3;
@@ -327,35 +407,39 @@ void ViewProviderMeshDuplicatedPoints::attach(App::DocumentObject* pcFeat)
 
     // Draw points
     SoSeparator* pointsep = new SoSeparator;
-    SoBaseColor * basecol = new SoBaseColor;
-    basecol->rgb.setValue( 1.0f, 0.5f, 0.0f );
+    SoBaseColor* basecol = new SoBaseColor;
+    basecol->rgb.setValue(1.0F, 0.5F, 0.0F);
     pointsep->addChild(basecol);
     pointsep->addChild(pcCoords);
     pointsep->addChild(pcPoints);
     pcPointRoot->addChild(pointsep);
 
     // Draw markers
-    SoBaseColor * markcol = new SoBaseColor;
-    markcol->rgb.setValue( 1.0f, 1.0f, 0.0f );
+    SoBaseColor* markcol = new SoBaseColor;
+    markcol->rgb.setValue(1.0F, 1.0F, 0.0F);
     SoMarkerSet* marker = new SoMarkerSet;
-    marker->markerIndex=SoMarkerSet::PLUS_7_7;
+    marker->markerIndex = Gui::Inventor::MarkerBitmaps::getMarkerIndex(
+        "PLUS",
+        App::GetApplication()
+            .GetParameterGroupByPath("User parameter:BaseApp/Preferences/View")
+            ->GetInt("MarkerSize", 7));
     pointsep->addChild(markcol);
     pointsep->addChild(marker);
 
     addDisplayMaskMode(pcPointRoot, "Point");
 }
 
-void ViewProviderMeshDuplicatedPoints::showDefects(const std::vector<unsigned long>& inds)
+void ViewProviderMeshDuplicatedPoints::showDefects(const std::vector<Mesh::ElementIndex>& inds)
 {
-    Mesh::Feature* f = dynamic_cast<Mesh::Feature*>(pcObject);
-    const MeshCore::MeshKernel & rMesh = f->Mesh.getValue().getKernel();
+    Mesh::Feature* f = static_cast<Mesh::Feature*>(pcObject);
+    const MeshCore::MeshKernel& rMesh = f->Mesh.getValue().getKernel();
     pcCoords->point.deleteValues(0);
     pcCoords->point.setNum(inds.size());
     MeshCore::MeshPointIterator cP(rMesh);
-    unsigned long i = 0;
-    for ( std::vector<unsigned long>::const_iterator it = inds.begin(); it != inds.end(); ++it ) {
-        cP.Set(*it);
-        pcCoords->point.set1Value(i++,cP->x,cP->y,cP->z);
+    int i = 0;
+    for (Mesh::ElementIndex ind : inds) {
+        cP.Set(ind);
+        pcCoords->point.set1Value(i++, cP->x, cP->y, cP->z);
     }
 
     setDisplayMaskMode("Point");
@@ -365,8 +449,10 @@ void ViewProviderMeshDuplicatedPoints::showDefects(const std::vector<unsigned lo
 
 ViewProviderMeshDegenerations::ViewProviderMeshDegenerations()
 {
+    // NOLINTBEGIN
     pcLines = new SoLineSet;
     pcLines->ref();
+    // NOLINTEND
 }
 
 ViewProviderMeshDegenerations::~ViewProviderMeshDegenerations()
@@ -376,7 +462,7 @@ ViewProviderMeshDegenerations::~ViewProviderMeshDegenerations()
 
 void ViewProviderMeshDegenerations::attach(App::DocumentObject* pcFeat)
 {
-    ViewProviderDocumentObject::attach( pcFeat );
+    ViewProviderDocumentObject::attach(pcFeat);
 
     SoGroup* pcLineRoot = new SoGroup();
     pcDrawStyle->lineWidth = 3;
@@ -384,71 +470,81 @@ void ViewProviderMeshDegenerations::attach(App::DocumentObject* pcFeat)
 
     // Draw lines
     SoSeparator* linesep = new SoSeparator;
-    SoBaseColor * basecol = new SoBaseColor;
-    basecol->rgb.setValue( 1.0f, 0.5f, 0.0f );
+    SoBaseColor* basecol = new SoBaseColor;
+    basecol->rgb.setValue(1.0F, 0.5F, 0.0F);
     linesep->addChild(basecol);
     linesep->addChild(pcCoords);
     linesep->addChild(pcLines);
     pcLineRoot->addChild(linesep);
 
     // Draw markers
-    SoBaseColor * markcol = new SoBaseColor;
-    markcol->rgb.setValue( 1.0f, 1.0f, 0.0f );
+    SoBaseColor* markcol = new SoBaseColor;
+    markcol->rgb.setValue(1.0F, 1.0F, 0.0F);
     SoMarkerSet* marker = new SoMarkerSet;
-    marker->markerIndex=SoMarkerSet::PLUS_7_7;
+    marker->markerIndex = Gui::Inventor::MarkerBitmaps::getMarkerIndex(
+        "PLUS",
+        App::GetApplication()
+            .GetParameterGroupByPath("User parameter:BaseApp/Preferences/View")
+            ->GetInt("MarkerSize", 7));
     linesep->addChild(markcol);
     linesep->addChild(marker);
 
     addDisplayMaskMode(pcLineRoot, "Line");
 }
 
-void ViewProviderMeshDegenerations::showDefects(const std::vector<unsigned long>& inds)
+void ViewProviderMeshDegenerations::showDefects(const std::vector<Mesh::ElementIndex>& inds)
 {
-    Mesh::Feature* f = dynamic_cast<Mesh::Feature*>(pcObject);
-    const MeshCore::MeshKernel & rMesh = f->Mesh.getValue().getKernel();
+    Mesh::Feature* f = static_cast<Mesh::Feature*>(pcObject);
+    const MeshCore::MeshKernel& rMesh = f->Mesh.getValue().getKernel();
 
     pcCoords->point.deleteValues(0);
-    pcCoords->point.setNum(2*inds.size());
+    pcCoords->point.setNum(2 * inds.size());
     MeshCore::MeshFacetIterator cF(rMesh);
-    unsigned long i=0;
-    unsigned long j=0;
-    for (std::vector<unsigned long>::const_iterator it = inds.begin(); it != inds.end(); ++it) {
-        cF.Set(*it);
-        const MeshCore::MeshPoint& rE0 = cF->_aclPoints[0]; 
+    int i = 0;
+    int j = 0;
+    for (Mesh::ElementIndex ind : inds) {
+        cF.Set(ind);
+        const MeshCore::MeshPoint& rE0 = cF->_aclPoints[0];
         const MeshCore::MeshPoint& rE1 = cF->_aclPoints[1];
         const MeshCore::MeshPoint& rE2 = cF->_aclPoints[2];
 
         // check if the points are coincident
         if (rE0 == rE1 && rE0 == rE2) {
             // set a small tolerance to get a non-degenerated line
-            float eps = 0.005f;
+            float eps = 0.005F;
             Base::Vector3f cP1, cP2;
-            cP1.Set(rE1.x+eps,rE1.y+eps,rE1.z+eps);
-            cP2.Set(rE2.x-eps,rE2.y-eps,rE2.z-eps);
-            pcCoords->point.set1Value(i++,cP1.x,cP1.y,cP1.z);
-            pcCoords->point.set1Value(i++,cP2.x,cP2.y,cP2.z);
+            cP1.Set(rE1.x + eps, rE1.y + eps, rE1.z + eps);
+            cP2.Set(rE2.x - eps, rE2.y - eps, rE2.z - eps);
+            pcCoords->point.set1Value(i++, cP1.x, cP1.y, cP1.z);
+            pcCoords->point.set1Value(i++, cP2.x, cP2.y, cP2.z);
         }
         else if (rE0 == rE1) {
-            pcCoords->point.set1Value(i++,rE1.x,rE1.y,rE1.z);
-            pcCoords->point.set1Value(i++,rE2.x,rE2.y,rE2.z);
+            pcCoords->point.set1Value(i++, rE1.x, rE1.y, rE1.z);
+            pcCoords->point.set1Value(i++, rE2.x, rE2.y, rE2.z);
         }
         else if (rE1 == rE2) {
-            pcCoords->point.set1Value(i++,rE2.x,rE2.y,rE2.z);
-            pcCoords->point.set1Value(i++,rE0.x,rE0.y,rE0.z);
+            pcCoords->point.set1Value(i++, rE2.x, rE2.y, rE2.z);
+            pcCoords->point.set1Value(i++, rE0.x, rE0.y, rE0.z);
         }
         else if (rE2 == rE0) {
-            pcCoords->point.set1Value(i++,rE0.x,rE0.y,rE0.z);
-            pcCoords->point.set1Value(i++,rE1.x,rE1.y,rE1.z);
+            pcCoords->point.set1Value(i++, rE0.x, rE0.y, rE0.z);
+            pcCoords->point.set1Value(i++, rE1.x, rE1.y, rE1.z);
         }
         else {
-            for (int j=0; j<3; j++) {
-                Base::Vector3f cVec1 = cF->_aclPoints[(j+1)%3] - cF->_aclPoints[j];
-                Base::Vector3f cVec2 = cF->_aclPoints[(j+2)%3] - cF->_aclPoints[j];
+            for (int j = 0; j < 3; j++) {
+                Base::Vector3f cVec1 = cF->_aclPoints[(j + 1) % 3] - cF->_aclPoints[j];
+                Base::Vector3f cVec2 = cF->_aclPoints[(j + 2) % 3] - cF->_aclPoints[j];
 
                 // adjust the neighbourhoods and point indices
-                if (cVec1 * cVec2 < 0.0f) {
-                    pcCoords->point.set1Value(i++,cF->_aclPoints[(j+1)%3].x,cF->_aclPoints[(j+1)%3].y,cF->_aclPoints[(j+1)%3].z);
-                    pcCoords->point.set1Value(i++,cF->_aclPoints[(j+2)%3].x,cF->_aclPoints[(j+2)%3].y,cF->_aclPoints[(j+2)%3].z);
+                if (cVec1 * cVec2 < 0.0F) {
+                    pcCoords->point.set1Value(i++,
+                                              cF->_aclPoints[(j + 1) % 3].x,
+                                              cF->_aclPoints[(j + 1) % 3].y,
+                                              cF->_aclPoints[(j + 1) % 3].z);
+                    pcCoords->point.set1Value(i++,
+                                              cF->_aclPoints[(j + 2) % 3].x,
+                                              cF->_aclPoints[(j + 2) % 3].y,
+                                              cF->_aclPoints[(j + 2) % 3].z);
                     break;
                 }
             }
@@ -464,8 +560,10 @@ void ViewProviderMeshDegenerations::showDefects(const std::vector<unsigned long>
 
 ViewProviderMeshIndices::ViewProviderMeshIndices()
 {
+    // NOLINTBEGIN
     pcFaces = new SoFaceSet;
     pcFaces->ref();
+    // NOLINTEND
 }
 
 ViewProviderMeshIndices::~ViewProviderMeshIndices()
@@ -475,7 +573,7 @@ ViewProviderMeshIndices::~ViewProviderMeshIndices()
 
 void ViewProviderMeshIndices::attach(App::DocumentObject* pcFeat)
 {
-    ViewProviderDocumentObject::attach( pcFeat );
+    ViewProviderDocumentObject::attach(pcFeat);
 
     SoGroup* pcFaceRoot = new SoGroup();
 
@@ -483,49 +581,52 @@ void ViewProviderMeshIndices::attach(App::DocumentObject* pcFeat)
     pcFlatStyle->style = SoDrawStyle::FILLED;
     pcFaceRoot->addChild(pcFlatStyle);
 
-    SoShapeHints * flathints = new SoShapeHints;
-    flathints->vertexOrdering = SoShapeHints::COUNTERCLOCKWISE ;
+    SoShapeHints* flathints = new SoShapeHints;
+    flathints->vertexOrdering = SoShapeHints::COUNTERCLOCKWISE;
     flathints->shapeType = SoShapeHints::UNKNOWN_SHAPE_TYPE;
     pcFaceRoot->addChild(flathints);
 
     // Draw lines
     SoSeparator* linesep = new SoSeparator;
-    SoBaseColor * basecol = new SoBaseColor;
-    basecol->rgb.setValue( 1.0f, 0.5f, 0.0f );
+    SoBaseColor* basecol = new SoBaseColor;
+    basecol->rgb.setValue(1.0F, 0.5F, 0.0F);
     linesep->addChild(basecol);
     linesep->addChild(pcCoords);
     linesep->addChild(pcFaces);
     pcFaceRoot->addChild(linesep);
 
     // Draw markers
-    SoBaseColor * markcol = new SoBaseColor;
-    markcol->rgb.setValue( 1.0f, 1.0f, 0.0f );
+    SoBaseColor* markcol = new SoBaseColor;
+    markcol->rgb.setValue(1.0F, 1.0F, 0.0F);
     SoMarkerSet* marker = new SoMarkerSet;
-    marker->markerIndex=SoMarkerSet::PLUS_7_7;
+    marker->markerIndex = Gui::Inventor::MarkerBitmaps::getMarkerIndex(
+        "PLUS",
+        App::GetApplication()
+            .GetParameterGroupByPath("User parameter:BaseApp/Preferences/View")
+            ->GetInt("MarkerSize", 7));
     linesep->addChild(markcol);
     linesep->addChild(marker);
 
     addDisplayMaskMode(pcFaceRoot, "Face");
 }
 
-void ViewProviderMeshIndices::showDefects(const std::vector<unsigned long>& inds)
+void ViewProviderMeshIndices::showDefects(const std::vector<Mesh::ElementIndex>& inds)
 {
-    Mesh::Feature* f = dynamic_cast<Mesh::Feature*>(pcObject);
-    const MeshCore::MeshKernel & rMesh = f->Mesh.getValue().getKernel();
+    Mesh::Feature* f = static_cast<Mesh::Feature*>(pcObject);
+    const MeshCore::MeshKernel& rMesh = f->Mesh.getValue().getKernel();
 
     if (!inds.empty()) {
         pcCoords->point.deleteValues(0);
-        pcCoords->point.setNum(3*inds.size());
+        pcCoords->point.setNum(3 * inds.size());
         MeshCore::MeshFacetIterator cF(rMesh);
-        unsigned long i=0;
-        unsigned long j=0;
-        for (std::vector<unsigned long>::const_iterator it = inds.begin(); it != inds.end(); ++it) {
-            cF.Set(*it);
-            for (int k=0; k<3; k++) {
-                Base::Vector3f cP = cF->_aclPoints[k];
+        int i = 0;
+        int j = 0;
+        for (Mesh::ElementIndex ind : inds) {
+            cF.Set(ind);
+            for (auto cP : cF->_aclPoints) {
                 // move a bit in opposite normal direction to overlay the original faces
-                cP -= 0.001f * cF->GetNormal();
-                pcCoords->point.set1Value(i++,cP.x,cP.y,cP.z);
+                cP -= 0.001F * cF->GetNormal();
+                pcCoords->point.set1Value(i++, cP.x, cP.y, cP.z);
             }
             pcFaces->numVertices.set1Value(j++, 3);
         }
@@ -538,8 +639,10 @@ void ViewProviderMeshIndices::showDefects(const std::vector<unsigned long>& inds
 
 ViewProviderMeshSelfIntersections::ViewProviderMeshSelfIntersections()
 {
+    // NOLINTBEGIN
     pcLines = new SoLineSet;
     pcLines->ref();
+    // NOLINTEND
 }
 
 ViewProviderMeshSelfIntersections::~ViewProviderMeshSelfIntersections()
@@ -549,7 +652,7 @@ ViewProviderMeshSelfIntersections::~ViewProviderMeshSelfIntersections()
 
 void ViewProviderMeshSelfIntersections::attach(App::DocumentObject* pcFeat)
 {
-    ViewProviderDocumentObject::attach( pcFeat );
+    ViewProviderDocumentObject::attach(pcFeat);
 
     SoGroup* pcLineRoot = new SoGroup();
     pcDrawStyle->lineWidth = 3;
@@ -557,51 +660,57 @@ void ViewProviderMeshSelfIntersections::attach(App::DocumentObject* pcFeat)
 
     // Draw lines
     SoSeparator* linesep = new SoSeparator;
-    SoBaseColor * basecol = new SoBaseColor;
-    basecol->rgb.setValue( 1.0f, 0.5f, 0.0f );
+    SoBaseColor* basecol = new SoBaseColor;
+    basecol->rgb.setValue(1.0F, 0.5F, 0.0F);
     linesep->addChild(basecol);
     linesep->addChild(pcCoords);
     linesep->addChild(pcLines);
     pcLineRoot->addChild(linesep);
 
     // Draw markers
-    SoBaseColor * markcol = new SoBaseColor;
-    markcol->rgb.setValue( 1.0f, 1.0f, 0.0f );
+    SoBaseColor* markcol = new SoBaseColor;
+    markcol->rgb.setValue(1.0F, 1.0F, 0.0F);
     SoMarkerSet* marker = new SoMarkerSet;
-    marker->markerIndex=SoMarkerSet::PLUS_7_7;
+    marker->markerIndex = Gui::Inventor::MarkerBitmaps::getMarkerIndex(
+        "PLUS",
+        App::GetApplication()
+            .GetParameterGroupByPath("User parameter:BaseApp/Preferences/View")
+            ->GetInt("MarkerSize", 7));
     linesep->addChild(markcol);
     linesep->addChild(marker);
 
     addDisplayMaskMode(pcLineRoot, "Line");
 }
 
-void ViewProviderMeshSelfIntersections::showDefects(const std::vector<unsigned long>& indices)
+void ViewProviderMeshSelfIntersections::showDefects(const std::vector<Mesh::ElementIndex>& indices)
 {
-    if (indices.size() % 2 != 0)
+    if (indices.size() % 2 != 0) {
         return;
-    Mesh::Feature* f = dynamic_cast<Mesh::Feature*>(pcObject);
-    const MeshCore::MeshKernel & rMesh = f->Mesh.getValue().getKernel();
+    }
+    Mesh::Feature* f = static_cast<Mesh::Feature*>(pcObject);
+    const MeshCore::MeshKernel& rMesh = f->Mesh.getValue().getKernel();
     MeshCore::MeshEvalSelfIntersection eval(rMesh);
-  
-    std::vector<std::pair<unsigned long, unsigned long> > intersection;
-    std::vector<unsigned long>::const_iterator it;
-    for (it = indices.begin(); it != indices.end(); ) {
-        unsigned long id1 = *it; ++it;
-        unsigned long id2 = *it; ++it;
-        intersection.push_back(std::make_pair
-            <unsigned long, unsigned long>(id1,id2));
+
+    std::vector<std::pair<Mesh::ElementIndex, Mesh::ElementIndex>> intersection;
+    std::vector<Mesh::ElementIndex>::const_iterator it;
+    for (it = indices.begin(); it != indices.end();) {
+        Mesh::ElementIndex id1 = *it;
+        ++it;
+        Mesh::ElementIndex id2 = *it;
+        ++it;
+        intersection.emplace_back(id1, id2);
     }
 
-    std::vector<std::pair<Base::Vector3f, Base::Vector3f> > lines;
+    std::vector<std::pair<Base::Vector3f, Base::Vector3f>> lines;
     eval.GetIntersections(intersection, lines);
 
     pcCoords->point.deleteValues(0);
-    pcCoords->point.setNum(2*lines.size());
-    unsigned long i=0;
-    unsigned long j=0;
-    for (std::vector<std::pair<Base::Vector3f, Base::Vector3f> >::const_iterator it = lines.begin(); it != lines.end(); ++it) {
-        pcCoords->point.set1Value(i++,it->first.x,it->first.y,it->first.z);
-        pcCoords->point.set1Value(i++,it->second.x,it->second.y,it->second.z);
+    pcCoords->point.setNum(2 * lines.size());
+    int i = 0;
+    int j = 0;
+    for (const auto& line : lines) {
+        pcCoords->point.set1Value(i++, line.first.x, line.first.y, line.first.z);
+        pcCoords->point.set1Value(i++, line.second.x, line.second.y, line.second.z);
         pcLines->numVertices.set1Value(j++, 2);
     }
 
@@ -612,8 +721,10 @@ void ViewProviderMeshSelfIntersections::showDefects(const std::vector<unsigned l
 
 ViewProviderMeshFolds::ViewProviderMeshFolds()
 {
+    // NOLINTBEGIN
     pcFaces = new SoFaceSet;
     pcFaces->ref();
+    // NOLINTEND
 }
 
 ViewProviderMeshFolds::~ViewProviderMeshFolds()
@@ -623,7 +734,7 @@ ViewProviderMeshFolds::~ViewProviderMeshFolds()
 
 void ViewProviderMeshFolds::attach(App::DocumentObject* pcFeat)
 {
-    ViewProviderDocumentObject::attach( pcFeat );
+    ViewProviderDocumentObject::attach(pcFeat);
 
     SoGroup* pcFaceRoot = new SoGroup();
 
@@ -631,48 +742,51 @@ void ViewProviderMeshFolds::attach(App::DocumentObject* pcFeat)
     pcFlatStyle->style = SoDrawStyle::FILLED;
     pcFaceRoot->addChild(pcFlatStyle);
 
-    SoShapeHints * flathints = new SoShapeHints;
-    flathints->vertexOrdering = SoShapeHints::COUNTERCLOCKWISE ;
+    SoShapeHints* flathints = new SoShapeHints;
+    flathints->vertexOrdering = SoShapeHints::COUNTERCLOCKWISE;
     flathints->shapeType = SoShapeHints::UNKNOWN_SHAPE_TYPE;
     pcFaceRoot->addChild(flathints);
 
     // Draw lines
     SoSeparator* linesep = new SoSeparator;
-    SoBaseColor * basecol = new SoBaseColor;
-    basecol->rgb.setValue( 1.0f, 0.0f, 0.0f );
+    SoBaseColor* basecol = new SoBaseColor;
+    basecol->rgb.setValue(1.0F, 0.0F, 0.0F);
     linesep->addChild(basecol);
     linesep->addChild(pcCoords);
     linesep->addChild(pcFaces);
     pcFaceRoot->addChild(linesep);
 
     // Draw markers
-    SoBaseColor * markcol = new SoBaseColor;
-    markcol->rgb.setValue( 1.0f, 1.0f, 0.0f );
+    SoBaseColor* markcol = new SoBaseColor;
+    markcol->rgb.setValue(1.0F, 1.0F, 0.0F);
     SoMarkerSet* marker = new SoMarkerSet;
-    marker->markerIndex=SoMarkerSet::PLUS_7_7;
+    marker->markerIndex = Gui::Inventor::MarkerBitmaps::getMarkerIndex(
+        "PLUS",
+        App::GetApplication()
+            .GetParameterGroupByPath("User parameter:BaseApp/Preferences/View")
+            ->GetInt("MarkerSize", 7));
     linesep->addChild(markcol);
     linesep->addChild(marker);
 
     addDisplayMaskMode(pcFaceRoot, "Face");
 }
 
-void ViewProviderMeshFolds::showDefects(const std::vector<unsigned long>& inds)
+void ViewProviderMeshFolds::showDefects(const std::vector<Mesh::ElementIndex>& inds)
 {
-    Mesh::Feature* f = dynamic_cast<Mesh::Feature*>(pcObject);
-    const MeshCore::MeshKernel & rMesh = f->Mesh.getValue().getKernel();
+    Mesh::Feature* f = static_cast<Mesh::Feature*>(pcObject);
+    const MeshCore::MeshKernel& rMesh = f->Mesh.getValue().getKernel();
 
     pcCoords->point.deleteValues(0);
-    pcCoords->point.setNum(3*inds.size());
+    pcCoords->point.setNum(3 * inds.size());
     MeshCore::MeshFacetIterator cF(rMesh);
-    unsigned long i=0;
-    unsigned long j=0;
-    for (std::vector<unsigned long>::const_iterator it = inds.begin(); it != inds.end(); ++it) {
-        cF.Set(*it);
-        for (int k=0; k<3; k++) {
-            Base::Vector3f cP = cF->_aclPoints[k];
+    int i = 0;
+    int j = 0;
+    for (Mesh::ElementIndex ind : inds) {
+        cF.Set(ind);
+        for (auto cP : cF->_aclPoints) {
             // move a bit in normal direction to overlay the original faces
-            cP += 0.001f * cF->GetNormal();
-            pcCoords->point.set1Value(i++,cP.x,cP.y,cP.z);
+            cP += 0.001F * cF->GetNormal();
+            pcCoords->point.set1Value(i++, cP.x, cP.y, cP.z);
         }
         pcFaces->numVertices.set1Value(j++, 3);
     }
