@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (c) 2008 Jrgen Riegel (juergen.riegel@web.de)              *
+ *   Copyright (c) 2008 Jürgen Riegel <juergen.riegel@web.de>              *
  *                                                                         *
  *   This file is part of the FreeCAD CAx development system.              *
  *                                                                         *
@@ -20,44 +20,30 @@
  *                                                                         *
  ***************************************************************************/
 
-
 #include "PreCompiled.h"
-#ifndef _PreComp_
-# include <Python.h>
-#endif
 
 #include <Base/Console.h>
 #include <Base/Interpreter.h>
- 
+#include <Base/PyObjectBase.h>
 
-extern struct PyMethodDef MeshPart_methods[];
-
-PyDoc_STRVAR(module_MeshPart_doc,
-"This module is the MeshPart module.");
 
+namespace MeshPart
+{
+extern PyObject* initModule();
+}
 
 /* Python entry */
-extern "C" {
-void AppMeshPartExport initMeshPart()
+PyMOD_INIT_FUNC(MeshPart)
 {
     // load dependent module
     try {
         Base::Interpreter().loadModule("Part");
-        //Base::Interpreter().loadModule("Mesh");
     }
-    catch(const Base::Exception& e) {
+    catch (const Base::Exception& e) {
         PyErr_SetString(PyExc_ImportError, e.what());
-        return;
+        PyMOD_Return(nullptr);
     }
-    Py_InitModule3("MeshPart", MeshPart_methods, module_MeshPart_doc);   /* mod name, table ptr */
+    PyObject* mod = MeshPart::initModule();
     Base::Console().Log("Loading MeshPart module... done\n");
-
-
-    // NOTE: To finish the initialization of our own type objects we must
-    // call PyType_Ready, otherwise we run into a segmentation fault, later on.
-    // This function is responsible for adding inherited slots from a type's base class.
- 
-    //MeshPart::FeatureViewPart        ::init();
+    PyMOD_Return(mod);
 }
-
-} // extern "C"
